diff --git a/modules/audio_filter/equalizer.c b/modules/audio_filter/equalizer.c
index e10696f..5d4d1b5 100644
--- a/modules/audio_filter/equalizer.c
+++ b/modules/audio_filter/equalizer.c
@@ -70,6 +70,10 @@ static void Close( vlc_object_t * );
 #define PREAMP_TEXT N_("Global gain" )
 #define PREAMP_LONGTEXT N_("Set the global gain in dB (-20 ... 20)." )
 
+#define PRESETS_TEXT N_( "Equalizer presets values" )
+#define PRESETS_LONGTEXT N_( "Presets string to derive available presets " \
+         "from.  Meant for internal use" )
+
 vlc_module_begin ()
     set_description( N_("Equalizer with 10 bands") )
     set_shortname( N_("Equalizer" ) )
@@ -84,8 +88,11 @@ vlc_module_begin ()
                 BANDS_LONGTEXT, true )
     add_bool( "equalizer-2pass", 0, NULL, TWOPASS_TEXT,
               TWOPASS_LONGTEXT, true )
-    add_float( "equalizer-preamp", 12.0, NULL, PREAMP_TEXT,
+    add_float( "equalizer-preamp", EQZ_DEFAULT_PREAMP, NULL, PREAMP_TEXT,
                PREAMP_LONGTEXT, true )
+    add_string( "equalizer-presets", NULL, NULL, PRESETS_TEXT,
+                PRESETS_LONGTEXT, true);
+         change_internal()
     set_callbacks( Open, Close )
     add_shortcut( "equalizer" )
 vlc_module_end ()
@@ -505,30 +512,34 @@ static int PresetCallback( vlc_object_t *p_this, char const *psz_cmd,
     aout_filter_sys_t *p_sys = (aout_filter_sys_t *)p_data;
     aout_instance_t *p_aout = (aout_instance_t *)p_this;
 
+    bool b_i_locked_eqz = !eqz_access_trylock();
+
     char *psz_preset = newval.psz_string;
 
     if( !*psz_preset || p_sys->i_band != 10 )
         return VLC_SUCCESS;
 
-    for( unsigned i = 0; eqz_preset_10b[i] != NULL; i++ )
+    for( unsigned i = 0; i != eqz_lengthof_presets(); i++ )
     {
-        if( !strcasecmp( eqz_preset_10b[i]->psz_name, psz_preset ) )
+        if( !strcasecmp( eqz_preset_10b[i].psz_name, psz_preset ) )
         {
             char *psz_newbands = NULL;
 
-            p_sys->f_gamp *= pow( 10, eqz_preset_10b[i]->f_preamp / 20.0 );
+            p_sys->f_gamp *= pow( 10, eqz_preset_10b[i].f_preamp / 20.0 );
             for( int j = 0; j < p_sys->i_band; j++ )
             {
                 lldiv_t d;
                 char *psz;
 
-                p_sys->f_amp[j] = EqzConvertdB( eqz_preset_10b[i]->f_amp[j] );
-                d = lldiv( eqz_preset_10b[i]->f_amp[j] * 10000000, 10000000 );
+                p_sys->f_amp[j] = EqzConvertdB( eqz_preset_10b[i].f_amp[j] );
+                d = lldiv( eqz_preset_10b[i].f_amp[j] * 10000000, 10000000 );
                 if( asprintf( &psz, "%s %lld.%07llu",
                               psz_newbands ? psz_newbands : "",
                               d.quot, d.rem ) == -1 )
                 {
                     free( psz_newbands );
+                    if (b_i_locked_eqz)
+                        eqz_access_unlock();
                     return VLC_ENOMEM;
                 }
                 psz_newbands = psz;
@@ -537,21 +548,25 @@ static int PresetCallback( vlc_object_t *p_this, char const *psz_cmd,
             {
                 var_SetString( p_aout, "equalizer-bands", psz_newbands );
                 var_SetFloat( p_aout, "equalizer-preamp",
-                              eqz_preset_10b[i]->f_preamp );
+                              eqz_preset_10b[i].f_preamp );
                 free( psz_newbands );
             }
             else
             {
                 p_sys->psz_newbands = psz_newbands;
-                p_sys->f_newpreamp = eqz_preset_10b[i]->f_preamp;
+                p_sys->f_newpreamp = eqz_preset_10b[i].f_preamp;
             }
+            if (b_i_locked_eqz)
+                eqz_access_unlock();
             return VLC_SUCCESS;
         }
     }
     msg_Err( p_aout, "equalizer preset '%s' not found", psz_preset );
     msg_Info( p_aout, "full list:" );
-    for( unsigned i = 0; eqz_preset_10b[i] != NULL; i++ )
-         msg_Info( p_aout, "  - '%s'", eqz_preset_10b[i]->psz_name );
+    for( unsigned i = 0; i != eqz_lengthof_presets(); i++ )
+         msg_Info( p_aout, "  - '%s'", eqz_preset_10b[i].psz_name );
+    if (b_i_locked_eqz)
+        eqz_access_unlock();
     return VLC_SUCCESS;
 }
 
diff --git a/modules/audio_filter/equalizer_presets.h b/modules/audio_filter/equalizer_presets.h
index 1e52d94..fb2e669 100644
--- a/modules/audio_filter/equalizer_presets.h
+++ b/modules/audio_filter/equalizer_presets.h
@@ -5,6 +5,7 @@
  * $Id$
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
+ *          Dave Andrews <jetdog330@hotmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,8 +30,14 @@
  */
 
 #define EQZ_BANDS_MAX 10
-
 #define NB_PRESETS 18
+#define EQZ_DEFAULT_PREAMP 12.0
+
+/* Internal-ONLY defines */
+#define NB_EMPTY_PRESET_SLOTS 3
+#define NB_DEFAULT_PRESETS NB_PRESETS
+#define PRESET_CONFIG_PARAMS 4
+
 static const char *const preset_list[] = {
     "flat", "classical", "club", "dance", "fullbass", "fullbasstreble",
     "fulltreble", "headphones","largehall", "live", "party", "pop", "reggae",
@@ -53,12 +60,12 @@ typedef struct
 
 static const eqz_preset_t eqz_preset_flat_10b=
 {
-    "flat", 10, 12.0,
+    "flat", 10, EQZ_DEFAULT_PREAMP,
     { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
 };
 static const eqz_preset_t eqz_preset_classical_10b=
 {
-    "classical", 10, 12.0,
+    "classical", 10, EQZ_DEFAULT_PREAMP,
     { -1.11022e-15, -1.11022e-15, -1.11022e-15, -1.11022e-15, -1.11022e-15, -1.11022e-15, -7.2, -7.2, -7.2, -9.6 }
 };
 static const eqz_preset_t eqz_preset_club_10b=
@@ -143,7 +150,7 @@ static const eqz_preset_t eqz_preset_techno_10b=
     { 8, 5.6, -1.11022e-15, -5.6, -4.8, -1.11022e-15, 8, 9.6, 9.6, 8.8 }
 };
 
-static const eqz_preset_t *eqz_preset_10b[] =
+static const eqz_preset_t * eqz_default_preset_10b[] =
 {
     &eqz_preset_flat_10b,
     &eqz_preset_classical_10b,
@@ -166,5 +173,351 @@ static const eqz_preset_t *eqz_preset_10b[] =
     NULL
 };
 
+/* Internal typedef's */
+typedef eqz_preset_t __t_eqz;
+
+/* Externally Accessible Array of Presets */
+/* Note: you need to lock eqz_access with function calls before using this */
+static const __t_eqz * eqz_preset_10b = NULL;
+
+/* Prototypes - for externally accessible functions */
+static void __eqz_access_lock();
+static int __eqz_access_trylock();
+static void __eqz_access_unlock();
+static void __eqz_init_presets( vlc_object_t * );
+static void __eqz_reload_config_data( vlc_object_t * );
+static void __eqz_reset_presets( vlc_object_t * );
+inline static const unsigned __eqz_lengthof_presets();
+
+/* Prototypes - for Internal-only functions */
+static int __eqz_load_presets( vlc_object_t *, char *);
+static void __eqz_save_presets( vlc_object_t * );
+static void __eqz_default_presets();
+
+/* Internal Data */
+static __t_eqz * __p_preset_array;
+static __t_eqz * __p_preset_end;
+static bool __eqz_presets_ready = false;
+static vlc_mutex_t __lock_presets;
+
+
+#define eqz_lengthof_presets() __eqz_lengthof_presets()
+inline static const unsigned __eqz_lengthof_presets()
+{
+    return (__p_preset_end - eqz_preset_10b);
+}
+
+/* Should-Be External Functions (note: prototypes are above) */
+#define eqz_access_lock() __eqz_access_lock()
+static void __eqz_access_lock()
+{
+    vlc_mutex_lock( &__lock_presets );
+    if ( !__eqz_presets_ready )
+    {
+        __eqz_default_presets();
+        __eqz_presets_ready = true;
+    }
+}
+
+#define eqz_access_trylock() __eqz_access_trylock()
+static int __eqz_access_trylock ()
+{
+    int ret = vlc_mutex_trylock( &__lock_presets );
+    if ( ret == 0 && !__eqz_presets_ready )
+    {
+        __eqz_default_presets();
+        __eqz_presets_ready = true;
+    }
+    return ret;
+}
+#define eqz_access_unlock() __eqz_access_unlock()
+static void __eqz_access_unlock()
+{
+    vlc_mutex_unlock( &__lock_presets );
+}
+
+/* Initializes Presets from config Data, if not already been initialized */
+#define eqz_init_presets(a) __eqz_init_presets(VLC_OBJECT(a))
+static void __eqz_init_presets( vlc_object_t * your_thread )
+{
+    msg_Dbg(your_thread, "==============INITIALIZING PRESETS===============");
+
+    /* Access the config data */
+    char * psz_presets = config_GetPsz(your_thread, "equalizer-presets");
+
+    bool b_i_locked_eqz = !eqz_access_trylock();
+
+    if (psz_presets == NULL)
+    {
+        __eqz_default_presets();
+    }
+    else
+    {
+        msg_Dbg(your_thread, "==============LOADING PRESETS===============");
+        __eqz_load_presets(your_thread, psz_presets);
+    }
+
+    if (b_i_locked_eqz)
+       eqz_access_unlock();
+
+    free (psz_presets);
+
+
+}
+
+/* Reloads equalizer config data from file */
+#define eqz_reload_presets(a) __eqz_reload_presets(VLC_OBJECT(a))
+static void __eqz_reload_presets( vlc_object_t * your_thread )
+{
+    bool b_i_locked_eqz = !eqz_access_trylock();
+
+    if (b_i_locked_eqz)
+       eqz_access_unlock();
+}
+
+/* Resets the equalizer config data to defaults - also saves to config */
+#define eqz_reset_presets(a) __eqz_reset_presets(VLC_OBJECT(a))
+static void __eqz_reset_presets( vlc_object_t * your_thread )
+{
+    bool b_i_locked_eqz = !eqz_access_trylock();
+
+    __eqz_default_presets();
+
+    if (b_i_locked_eqz)
+       eqz_access_unlock();
+}
+
+/* Internal Functions */
+static int __eqz_load_presets( vlc_object_t * your_thread, char * psz_presets )
+{    
+    /* For each preset in the config data... */
+    /* For future reference, this parser can theoretically work for */
+    /* Any number of equalizer bands - though, it is capped for now */
+    /* Note: format is: */
+/* name:simple_name:preamp:1 2 3 4 5 6 7 8 9 10<:start over again><:> */
+
+    /* Parse how many ':' 's we have in advance - saves on realloc()'s */
+    {
+        unsigned u_count_col = 0;
+        bool b_escape = false;
+        for (char * psz_i = psz_presets ; *psz_i != '\0' ; psz_i++)
+        {
+            if (*psz_i == '\\')
+            {
+                b_escape = !b_escape;
+            }
+            else if (*psz_i == ':')
+            {
+                if (!b_escape) /* If this entry is not escaped */
+                {
+                    u_count_col++;
+                }
+            }
+            else
+            {
+                b_escape = false;
+            }
+        }
+
+        __p_preset_array = (__t_eqz *) malloc( sizeof( __t_eqz ) *
+                ( NB_EMPTY_PRESET_SLOTS +
+                  ( ( u_count_col + 1 ) / PRESET_CONFIG_PARAMS ) ) );
+        /* Note, the above '+1' allows for an optional trailing ':' */
+        /* Not required; but if provided, no extra memory will be allocated */
+
+        if (__p_preset_array == NULL)
+        {
+            return -1;
+        }
+    }
+
+    eqz_preset_10b = ( const __t_eqz * ) __p_preset_array + NB_EMPTY_PRESET_SLOTS;
+    __t_eqz * p_i = ( __t_eqz * ) eqz_preset_10b;
+    __p_preset_end = ( __t_eqz * ) eqz_preset_10b + NB_DEFAULT_PRESETS;
+
+    char * end_item = psz_presets;
+    bool b_final_item = false;
+
+    /* For each preset entry */
+    while (1)
+    {
+        char * psz_item_start;
+        unsigned int u_i = 0;
+
+        /* For each item in the entry */
+        while (u_i < PRESET_CONFIG_PARAMS)
+        {
+            psz_item_start = end_item;
+            unsigned int u_item_length = 0;
+            bool b_escape = 0;
+            /* For each character in that item */
+            while(1)
+            {
+
+                if ( *end_item == '\0' )
+                {
+                    u_item_length++;
+                    if ( u_i == PRESET_CONFIG_PARAMS - 1)
+                    {
+                        b_final_item = true;
+                        break;
+                    }
+
+                    /* Errornous case : entry is not complete */
+                    msg_Dbg( your_thread, "Equalizer-Presets: invalid preset " \
+                            "encountered while parsing config file - omitting"
+                            );
+                    return -1;
+                }
+                if ( *end_item == '\\' )
+                {
+                    if ( ( b_escape = !b_escape ) == false)
+                    {
+                        u_item_length++;
+                    }
+                }
+                else if ( *end_item == ':' )
+                {
+                    if ( !b_escape )
+                    {
+                        u_item_length++;
+                        *end_item = '\0';
+                        break;
+                    }
+                }
+                else
+                {
+                    b_escape = 0;
+                }
+                end_item++;
+            }
+            /* End_item now contains the offset of our old ':' (has \0 now )*/
+            switch ( u_i )
+            {
+                case 0:
+                {
+                    /* Copy the name , exclude '/'s , and save pointer */
+                    p_i->psz_name = ( const char * ) malloc( u_item_length );
+                    if (p_i->psz_name == NULL)
+                    {
+                        return -1; /* memory error */
+                    }
+                    char * psz_name_src = psz_item_start;
+                    char * psz_name_dst = ( char * ) p_i->psz_name;
+                    bool b_escaped = false;
+                    do
+                    {
+                        if ( *psz_name_src == '\\' )
+                        {
+                            /* If character is an escaping one (ie '/' #1) */
+                            if ( ( b_escaped = !b_escaped ) == true )
+                            {
+                                psz_name_dst++;
+                                continue; /* Skip this character */
+                            }
+                        }
+                        else
+                        {
+                            b_escaped = false;
+                        }
+                        *psz_name_dst = *psz_name_src;
+                        psz_name_dst++;
+                    } while ( *psz_name_src++ != '\0' );
+
+                    break;
+                }
+                case 1:
+                    ;
+                    break;
+                case 2:
+                    /* Get the Preamp String -> if missing, we use default */
+                    if (sscanf (psz_item_start, "%f", &(p_i->f_preamp)) != 1)
+                    {
+                        p_i->f_preamp = EQZ_DEFAULT_PREAMP;
+                        /* TODO: msg a debug error */
+                    }
+                    ;
+                    break;
+                case 3:
+                {
+                    /* Tokenize the line by spaces - thread _safe_ */
+                    char * psz_resume_token;
+                    char * psz_band_token = strtok_r( psz_item_start, " ",
+                                                     &psz_resume_token );
+                    unsigned int u_current_band = 0;
+                    while ( psz_band_token != NULL &&
+                           u_current_band < EQZ_BANDS_MAX )
+                    {
+                        if ( sscanf ( psz_band_token, "%f",
+                                    &( p_i->f_amp[u_current_band] ) ) != 1 )
+                        {
+                            p_i->f_amp[u_current_band] = 0;
+                        }
+                        psz_band_token = strtok_r( NULL, " ",
+                                                   &psz_resume_token );
+                        u_current_band++;
+                    }
+
+                    if ( u_current_band < EQZ_BANDS_MAX )
+                    {
+                        msg_Dbg( your_thread, "Equalizer-Preset Loader: " \
+                                "Missing bands in your stored presets.  " \
+                                "They are being replaced with Zero's" );
+                        /* Fill the band with any missing values */
+                        do
+                        {
+                            p_i->f_amp[u_current_band] = 0;
+                        } while ( ++u_current_band < EQZ_BANDS_MAX );
+                    }
+
+                    break;
+                }
+            }
+
+            if (b_final_item)
+            {
+                return 0;
+            }
+
+            end_item++;
+            u_i++;
+        }
+    }
+}
+static void __eqz_save_presets( vlc_object_t * your_thread )
+{
+    config_PutPsz(your_thread, "equalizer-presets", "my preset");
+    config_SaveConfigFile( your_thread, "equalizer" );
+}
+
+static void __eqz_default_presets()
+{
+    __eqz_presets_ready = false;
+
+    // Set the Equalizer Presets from our config file
+    if (__p_preset_array != NULL)
+    {
+        free(__p_preset_array);
+    }
+        __p_preset_array = (__t_eqz *) malloc(
+            sizeof( __t_eqz ) * ( NB_EMPTY_PRESET_SLOTS +
+                                 NB_DEFAULT_PRESETS ) );
+    
+    eqz_preset_10b = ( const __t_eqz * ) __p_preset_array + NB_EMPTY_PRESET_SLOTS;
+
+    __t_eqz * p_i = ( __t_eqz * ) eqz_preset_10b;
+    __p_preset_end = ( __t_eqz * ) eqz_preset_10b + NB_DEFAULT_PRESETS;
+
 
+    // Load the Defaults we want
+    {
+        const eqz_preset_t * * p_j = eqz_default_preset_10b;
+        while ( p_i != __p_preset_end )
+        {
+            // Copy the default presets from the static Memory
+            memcpy( p_i++ , *p_j++ , sizeof( __t_eqz ) );
+        }
+    }
 
+    __eqz_presets_ready = true;
+}
diff --git a/modules/gui/qt4/components/extended_panels.cpp b/modules/gui/qt4/components/extended_panels.cpp
index 464c27f..4b5b1ad 100644
--- a/modules/gui/qt4/components/extended_panels.cpp
+++ b/modules/gui/qt4/components/extended_panels.cpp
@@ -827,6 +827,9 @@ Equalizer::Equalizer( intf_thread_t *_p_intf, QWidget *_parent ) :
     ui.setupUi( this );
     ui.preampLabel->setFont( smallFont );
 
+    /* Prepare the equalizer */
+    eqz_init_presets(_p_intf);
+
     /* Setup of presetsComboBox */
     presetsComboBox = ui.presetsCombo;
     CONNECT( presetsComboBox, currentIndexChanged( int ),
@@ -1010,7 +1013,9 @@ void Equalizer::updateUISliderValues( int i_preset )
 
     char *p = createValuesFromPreset( i_preset );
     char *psz = p;
-    float f_preamp = eqz_preset_10b[i_preset]->f_preamp;
+    eqz_access_lock(); /* Lock the Presets Mutex */
+    float f_preamp = eqz_preset_10b[i_preset].f_preamp;
+    eqz_access_unlock(); /* Unlock the Presets Mutex */
 
     if ( p )
     {
@@ -1040,9 +1045,14 @@ char * Equalizer::createValuesFromPreset( int i_preset )
     char *psz_values;
     QString values;
 
+   bool b_i_locked_eqz = !eqz_access_trylock();
+
     /* Create the QString in Qt */
-    for( int i = 0 ; i< BANDS ;i++ )
-        values += QString( " %1" ).arg( eqz_preset_10b[i_preset]->f_amp[i] );
+    for( int i = 0 ; i < eqz_lengthof_presets() ;i++ )
+        values += QString( " %1" ).arg( eqz_preset_10b[i_preset].f_amp[i] );
+
+   if (b_i_locked_eqz)
+        eqz_access_unlock();
 
     /* Convert it to char * */
     if( !asprintf( &psz_values, "%s", values.toAscii().constData() ) )
@@ -1053,6 +1063,7 @@ char * Equalizer::createValuesFromPreset( int i_preset )
 
 void Equalizer::setCorePreset( int i_preset )
 {
+    eqz_access_lock();
     char *psz_values = createValuesFromPreset( i_preset );
     if( !psz_values ) return ;
 
@@ -1064,14 +1075,15 @@ void Equalizer::setCorePreset( int i_preset )
 
         var_SetString( p_aout, "equalizer-bands", psz_values );
         var_SetFloat( p_aout, "equalizer-preamp",
-                      eqz_preset_10b[i_preset]->f_preamp );
+                      eqz_preset_10b[i_preset].f_preamp );
         addCallbacks( p_aout );
         vlc_object_release( p_aout );
     }
     config_PutPsz( p_intf, "equalizer-bands", psz_values );
     config_PutPsz( p_intf, "equalizer-preset", preset_list[i_preset] );
     config_PutFloat( p_intf, "equalizer-preamp",
-                    eqz_preset_10b[i_preset]->f_preamp );
+                    eqz_preset_10b[i_preset].f_preamp );
+    eqz_access_unlock();
     free( psz_values );
 }
 
